package adresar3;

import adresar2.Adresar;
import adresar2.ObjectReadAdresar;
import java.io.EOFException;
import java.io.IOException;
import java.util.NoSuchElementException;
import java.util.Scanner;

public class TransactionProcessor {

    /**
     * Instance pro manipulaci s datovým souborem.
     */
    private ObjectReadAdresar dataFile = new ObjectReadAdresar();
    /**
     * Vstup od uživatele z konzole.
     */
    private Scanner input = new Scanner(System.in);

    /**
     * Otevře soubor s daty.
     * @param fileName Jméno čteného souboru
     * @return true, pokud se soubor podařilo otevřít, false, pokud
     *         ne.
     */
    private boolean openFile(String fileName) {
        try {
            dataFile.openFile(fileName);
        } catch (IOException e) {
            System.err.println("Chyba otevírání souboru.");
            return false;
        }

        return true;
    }

    /**
     * Zavře soubor s daty.
     */
    private void closeFile() {
        try {
            dataFile.closeFile();
        } catch (IOException e) {
            System.err.println("Chyba zavírání souboru");
            System.exit(1);
        }
    }

    
    private void showRecord(Adresar adr) {
        System.out.println(adr.getFirstName() + " " + adr.getLastName());
        System.out.println(adr.getStreet() + " " + adr.getCisloPopisne());
        System.out.println(adr.getPSC() + " " + adr.getCity());
        System.out.println("Tel.: "+adr.getPhone());
        System.out.println();
    }
    /**
     * Provede akci reprezentovanou danou položkou menu.
     *
     * @param action
     *            položka menu odpovídající akci, která se má
     *            provést.
     */
    private void performAction(MenuOption action) {
        try {
            switch (action) {
                case PRINT:
                    dataFile.readRecords();
                    break;
                case SURNAME:
                    String surname;
                    int nalezeno = 0;
                    System.out.print("Zadejte hledané jméno: ");
                    surname = input.nextLine();
                    System.out.println("Nalezené záznamy:");
                    System.out.println();
                    while (true) {
                        try {
                            Adresar adr = dataFile.nextAdresar();
                            if (adr.getLastName().equals(surname)) {
                                showRecord(adr);
                                nalezeno++;
                            }
                        } catch (EOFException e) {
                            //konec souboru
                            System.out.printf("Nalezeno %d záznamů.%n", nalezeno);
                            break;
                        }
                    }
                    break;
                case FIND:
                    String find;
                    nalezeno = 0;
                    System.out.print("Zadejte hledaný řetězec: ");
                    find = input.nextLine();
                    System.out.println("Nalezené záznamy:");
                    System.out.println();
                    while (true) {
                        try {
                            Adresar adr = dataFile.nextAdresar();
                            String [] args = adr.getAttributesAsStrings();
                            for(String arg:args) {
                                if(arg.indexOf(find) != -1) {
                                    showRecord(adr);
                                    nalezeno++;
                                    break;
                                }
                            }
                        } catch (EOFException e) {
                            //konec souboru
                            System.out.printf("Nalezeno %d záznamů.%n", nalezeno);
                            break;
                        }
                    }
                    break;
                default:
                    System.out.println("Neplatná akce.");
                    break;
            }
        } catch (ClassNotFoundException e) {
            System.err.println(e);
        } catch (NumberFormatException e) {
            System.err.println("Špatný vstup.");
        } catch (IllegalArgumentException e) {
            System.err.println(e.getMessage());
        } catch (IOException e) {
            System.err.println("Chyba zápisu do souboru.");
        } catch (NoSuchElementException e) {
            System.err.println("Neplatný vstup. Zkuste to, prosím, znovu.");
            // načtení zbytku řádku
            input.nextLine();
        }
    }

    /**
     * Získá žádost od uživatele.
     *
     * @return položka menu vybraná uživatelem.
     */
    private MenuOption enterChoice() {
        int menuChoice = 1;

        System.out.println("\nZadejte svou volbu:");
        for (MenuOption mo : MenuOption.values()) {
            System.out.printf("%d - %s\n", mo.getValue(), mo.getTitle());
        }
        System.out.print("? ");

        try {
            menuChoice = input.nextInt();
            input.nextLine(); // ignoruji zbytek radky
        } catch (NoSuchElementException e) {
            System.err.println("Neplatný vstup.");
            System.exit(1);
        }

        return MenuOption.values()[menuChoice - 1];
    }

    /**
     * Získá od uživatele žádost a zpracuje ji.
     */
    public void processRequests(String fileName) {


        MenuOption choice = enterChoice();

        while (choice != MenuOption.END) {
            openFile(fileName);
            performAction(choice);
            closeFile();
            choice = enterChoice();
        }
    }
}
